/* VECTOR routine suite for variable-bit integer coding
 * Original vector and Elias coding Copyright (C) 1996 to Justin Zobel
 * This code, variable byte integers, and Golomb coding Copyright (C) 1996-8 
 * to Hugh Williams
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See Copyright.txt for further details. */

/* Code to test Golomb coding */

#include "def.h"

void 
main(int argc, char *argv[])
{
    BITVEC	vp2;
	int b;
	int x;

	/* Initialise the "b" lookup table for Golomb coding. */
	/* This is faster than calculating on-the-fly */
	initgolhash();
	
	/* Initialise the vector code */
	initcmp;
	
	/* Create a vector */
	initvec(&vp2, BIGVECLEN);
		
	/* Now, write the numbers 0 to 998 into the vector with weird b values */
	/* using Golomb coding */
	for (b=1000, x=0;x<999;x++, b--)
	{
		printf("Golomb putting: %d with b=%d\n", x, b);
		fflush(NULL);
		putgolomb(&vp2, x, b);
	}		

	/* Now, let's put some Elias stuff in for good measure */
	for (x=1;x<999;x++)
	{
		printf("Elias gamma putting: %d\n", x);
		putgamma(&vp2, x);
	}		

	for (x=1;x<999;x++)
	{
		printf("Elias delta putting: %d\n", x);
		putdelta(&vp2, x);
	}		

	/* Pad out the vector prior to writing */
	padvec(&vp2);		 

	/* reset the vector and read */	
	resetvec(&vp2);

	printf("-------------\n");

	/* Now, read the vector */
	for (b=1000, x=0;x<999;x++, b--)
	{
		printf("Golomb getting: %d\n", getgolomb(&vp2, b));
		fflush(NULL);
	}	

	for (x=1;x<999;x++)
		printf("Elias gamma getting: %d\n", getgamma(&vp2));

	for (x=1;x<999;x++)
		printf("Elias delta putting: %d\n", getdelta(&vp2));

	/* Free the vector memory */
	freevec(&vp2);
}
