/* Variable-byte integer coding
 * This code Copyright (C) 1996-8 to Hugh Williams
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See Copyright.txt for further details. */

#include "def.h"

/* 	Variable byte length reads and writes of integers */
/* 	We use a representation in which seven bits in each byte is used to code an
	integer, with the least significant bit set to 0 if this is the last byte,
	or to 1 if further bytes follow.
	In this way, we represent small integers efficiently; for example, we
	represent 135 in two bytes, since it lies in the range $[2^7\cdots 2^{14})$,
	as 00000011~00001110; this is read as 00000010000111 by removing the least
	significant bit from each byte and concatenating the remaining~14~bits. */

int vbyteread(FILE *fp)
{
	char tmp = 0x1;
	int val = 0;
	
		while((tmp & 0x1) == 0x1)
		{
			if (fread(&tmp, sizeof(char), 1, fp) == 0)
			{
				if (feof(fp))
					return(-1);
				else
					return(0);	
			}					
			val = (val << 7) + ((tmp >> 1) & 127);	
		}
	return(val);
}

int vbytewrite(int number, FILE *fp)
{
	char bytearray[4];
	char tmp = 0;
	int x, started = FALSE;
	int charswritten = 0;
	
		for(x=0;x<4;x++)
		{
			tmp = (number%128) << 1;
			bytearray[x] = tmp;
			number /= 128;
		}					
		for(x=3;x>0;x--)
		{
			if (bytearray[x] != 0 || started == TRUE)				
			{
				started = TRUE;
				bytearray[x] |= 0x1;	
				fwrite(&bytearray[x], sizeof(char), 1, fp);
				charswritten++;
			}
		}	
		bytearray[0] |= 0x0;
		fwrite(&bytearray[0], sizeof(char), 1, fp);
		charswritten++;
	return(charswritten);
}
